from base64 import b64decode
from functools import wraps

from django.contrib.auth import authenticate

from .utils import is_authenticated


def allow_basic_auth(view_func):
    @wraps(view_func)
    def wrapper(request, *args, **kwargs):
        if getattr(request, "user", None) is None:
            http_auth = request.META.get("HTTP_AUTHORIZATION")
            if http_auth and http_auth.startswith("Basic"):
                try:
                    _, token = http_auth.split()
                    username, password = b64decode(token).decode().split(":")
                    user = authenticate(request, username=username, password=password)
                    if user:
                        request.user = user
                except Exception:
                    pass
        return view_func(request, *args, **kwargs)

    return wrapper


def login_required(view_func):
    @wraps(view_func)
    def wrapper(request, *args, **kwargs):
        if is_authenticated(request):
            return view_func(request, *args, **kwargs)

    return wrapper


def wrap_result(key: str):
    def wrap_resolver(resolver):
        @wraps(resolver)
        def wrapper(*args, **kwargs):
            return {key: resolver(*args, **kwargs)}

        return wrapper

    return wrap_resolver



from ariadne import SchemaDirectiveVisitor
from django.core.exceptions import PermissionDenied
from graphql import default_field_resolver

from .utils import is_authenticated


class IsAuthenticatedDirective(SchemaDirectiveVisitor):
    def visit_field_definition(self, field, _):
        original_resolver = field.resolve or default_field_resolver

        def resolve_for_authenticated_user(parent, info, **kwargs):
            user = is_authenticated(
                info.context["request"], is_active=True, raise_exception=True
            )
            return original_resolver(parent, info, user=user, **kwargs)

        field.resolve = resolve_for_authenticated_user
        return field


class IsStaffDirective(SchemaDirectiveVisitor):
    def visit_field_definition(self, field, _):
        original_resolver = field.resolve or default_field_resolver

        def resolve_for_authenticated_user(parent, info, **kwargs):
            user = is_authenticated(info.context["request"], is_active=True)
            if user and user.is_staff:
                return original_resolver(parent, info, user=user, **kwargs)
            raise PermissionDenied()

        field.resolve = resolve_for_authenticated_user
        return field



from functools import partial, wraps
from typing import Callable, Union

from django.core.cache import cache as dj_cache
from django.core.cache.backends.base import DEFAULT_TIMEOUT

default = object()
delimiter = "::"


def cache_keygen(info, key):
    return "{}{delimiter}{}{delimiter}{}".format(
        info.path.typename,
        info.path.key,
        delimiter.join(str(k) for k in key)
        if isinstance(key, tuple) or isinstance(key, list)
        else key,
        delimiter=delimiter,
    )


def cache(key: Union[str, Callable], timeout=DEFAULT_TIMEOUT, version=None):
    def wrap_resolver(resolver: Callable):
        @wraps(resolver)
        def wrapper(parent, info, **kwargs):
            resolve = partial(resolver, parent, info, **kwargs)
            cache_key = key(parent, info, **kwargs) if callable(key) else key
            if cache_key is not None:
                cache_key = cache_keygen(info, cache_key)
                cached = dj_cache.get(cache_key, default, version)
                if cached != default:
                    return cached

                result = resolve()
                dj_cache.add(cache_key, result, timeout, version)
                return result
            return resolve()

        return wrapper

    return wrap_resolver



from .cache import cache
from .decorators import wrap_result
from .directives import IsAuthenticatedDirective, IsStaffDirective

__all__ = ["cache", "wrap_result", "IsAuthenticatedDirective", "IsStaffDirective"]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ariadne_django_ext']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'ariadne-django-ext',
    'version': '1.5',
    'description': 'Usable utility extensions for Ariadne & Django',
    'long_description': '<h1 align="center">\n  ariadne-django-ext\n</h1>\n\n<p align="center">\n  <a href="https://github.com/dulmandakh/ariadne-django-ext/">\n    <img src="https://img.shields.io/github/workflow/status/dulmandakh/ariadne-django-ext/CI?label=Test&logo=github&style=for-the-badge" alt="ci status">\n  </a>\n  <a href="https://pypi.org/project/ariadne-django-ext/">\n    <img src="https://img.shields.io/pypi/v/ariadne-django-ext?style=for-the-badge" alt="pypi link">\n  </a>\n  <a href="https://codecov.io/github/dulmandakh/ariadne-django-ext">\n    <img src="https://img.shields.io/codecov/c/github/dulmandakh/ariadne-django-ext?logo=codecov&style=for-the-badge" alt="codecov">\n  </a>\n  <br>\n  <a>\n    <img src="https://img.shields.io/pypi/pyversions/ariadne-django-ext?logo=python&style=for-the-badge" alt="supported python versions">\n  </a>\n  <a>\n    <img src="https://img.shields.io/pypi/djversions/ariadne-django-ext?logo=django&style=for-the-badge" alt="supported django versions">\n  </a>\n</p>\n\n<p align="center">\n  <a href="#installation">Installation</a> •\n  <a href="#contributing">Contributing</a> •\n  <a href="#license">License</a>\n</p>\n\n<p align="center">Custom, simple Django User model with email as username</p>\n\n## Installation\n\n```sh\npip install ariadne-django-ext\n```\n\n## Usage\n\n### cache\n\n**cache** decorator will cache a result returned from resolver using Django cache framework. You can it accepts same keyword arguments and passed down to Django cache.\n\nCache key must be either value or callable. Callable will receive same arguments as resolver then return cache key. Callable may return **None** to bypass the cache.\n\nIt uses **typename** and **key** from **info.path** as cache key prefix.\n\n```python\nfrom ariadne_django_ext import cache\n\n@cache(key=\'cache_key\')\ndef resolver(parent, info):\n    ...\n    return \'result\'\n\n```\n\n### wrap_result\n\n**wrap_result** decorator wraps return value of a resolver into dictionary with the key\n\n```python\nfrom ariadne_django_ext import wrap_result\n\n@wrap_result(key=\'result\')\ndef resolver(parent, info):\n    return \'result\'\n\n```\n\nAbove example will return following dict\n\n```json\n{ "result": "result" }\n```\n\n### isAuthenticated & isStaff directive\n\nA resolver will receive an authenticated user as keyword argument.\n\n```graphql\ndirective @isAuthenticated on FIELD_DEFINITION\n\ntype User {\n  id: ID\n  username: String\n  ipAddress: String @isAuthenticated\n}\n```\n\n```python\nfrom ariadne_django_ext import IsAuthenticatedDirective\n\nschema = make_executable_schema(\n  type_defs,\n  resolvers,\n  directives={"isAuthenticated": IsAuthenticatedDirective}\n)\n```\n\n## Contributing\n\nPull requests are welcome. For major changes, please open an issue first to discuss what you would like to change.\n\nPlease make sure to update tests as appropriate.\n\n## License\n\n[MIT License](https://choosealicense.com/licenses/mit/)\n\n```\n\n```\n',
    'author': 'Dulmandakh',
    'author_email': 'dulmandakh@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/dulmandakh/ariadne-django-ext',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)



from django.core.exceptions import PermissionDenied


def is_authenticated(request, is_active=True, raise_exception=True):
    user = getattr(request, "user", None)
    if user and user.is_authenticated and (not is_active or user.is_active):
        return user
    if raise_exception:
        raise PermissionDenied()
